
# q-expansion of newform 5292.2.l.c, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5292
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[2647, [1, 0]], [785, [-1, 1]], [1081, [0, -1]]]
aps_data = [[0, 0], [0, 0], [3, 0], [0, 0], [-3, 0], [-1, 1], [-6, 6], [0, -4], [3, 0], [0, 3], [0, 5], [0, -2], [-3, 3], [0, 1], [9, -9], [-6, 6], [0, 3], [-13, 13], [0, 7], [12, 0], [-10, 10], [-11, 11], [0, 9], [0, -6], [0, 11], [15, 0], [7, 0], [0, 12], [-2, 2], [-9, 9], [-16, 0], [21, 0], [-3, 0], [5, -5], [-15, 0], [-13, 0], [0, -13], [0, -20], [-9, 9], [9, -9], [12, -12], [-2, 0], [15, -15], [0, -11], [-6, 0], [-4, 4], [-17, 17], [0, -1], [27, 0], [13, 0], [0, -6], [-27, 27], [1, 0], [12, 0], [-9, 0], [21, 0], [6, -6], [0, 8], [-1, 0], [0, 3], [0, 5], [21, -21], [-20, 0], [0, -21], [-1, 1], [-21, 21], [-11, 11], [-23, 23], [0, 9], [0, -1], [3, 0], [0, 0], [13, 0], [-1, 0], [-16, 0], [-15, 0], [-15, 0], [0, 2], [-3, 0], [0, 23], [-9, 9], [0, -35], [24, -24], [34, 0], [35, -35], [-9, 9], [18, 0], [37, -37], [0, -3], [19, -19], [0, -12], [27, 0], [-32, 32], [-3, 3], [5, 0], [-36, 0], [39, 0], [-42, 42], [0, 8], [0, 34], [0, 13], [-30, 30], [0, -9], [15, -15], [0, 31], [-10, 10], [0, -15], [0, -6], [0, -39], [0, 35], [-41, 0], [-26, 26], [3, -3], [13, 0], [-16, 0], [33, 0], [41, -41], [0, 0], [21, 0], [0, -21], [0, 11], [0, -11], [-15, 15], [-36, 36], [-1, 1], [6, 0], [25, -25], [0, 0], [0, -37], [-23, 0], [16, -16], [9, -9], [-31, 0], [-10, 0], [27, 0], [-1, 1], [-18, 18], [0, -43], [9, -9], [6, -6], [-20, 0], [51, -51], [0, 19], [12, 0], [50, -50], [0, 9], [0, -13], [27, 0], [-41, 0], [0, 24], [23, 0], [30, 0], [-4, 0], [21, 0], [47, 0], [0, -45], [0, 16], [-27, 27], [34, 0], [0, 21], [27, -27], [-54, 0], [43, -43], [0, 0], [0, -57], [51, 0], [-8, 8], [1, 0], [-10, 0], [39, 0], [33, 0], [23, -23], [-21, 0], [-37, 0], [0, -55], [-15, 15], [0, 1], [-54, 54], [40, 0], [-1, 1], [1, -1], [0, -15], [13, -13], [0, 18], [0, 57], [-21, 0], [-62, 62], [1, 0], [-1, 0], [48, -48], [0, 14], [0, -45], [0, -20], [-15, 15], [60, -60], [0, -15], [0, 37], [-47, 47], [0, 57], [0, -24], [0, -45], [0, -25], [37, 0], [-23, 0], [-60, 0], [-42, 42], [0, -40], [51, 0], [0, 15], [0, 5], [0, -38], [57, -57], [0, -23], [21, -21], [0, -9], [23, -23], [0, -5], [-45, 0], [0, 48], [15, -15], [71, 0], [-31, 31], [-3, 0], [0, -56], [27, -27], [0, -47], [-63, 63], [-24, 24], [35, -35], [3, -3], [0, 13], [20, -20], [-5, 5], [0, 39], [0, 35], [-33, 0], [-35, 0], [0, 18], [45, -45], [-24, 0], [-15, 0], [0, -28], [0, 65], [0, 46], [-51, 51], [33, -33], [-32, 0], [0, 15], [0, 31], [-72, 0], [49, -49], [0, 18], [0, -3], [0, -49], [-9, 0], [0, 24], [-26, 26], [37, 0], [54, 0], [0, -34], [11, 0], [0, -9], [0, 11], [70, 0], [0, 69], [35, -35], [-42, 0], [0, 9], [31, -31], [0, 63], [-23, 0], [52, -52], [1, 0], [65, 0], [59, -59], [0, 8], [-21, 0], [35, 0], [0, -38], [21, -21], [0, 15], [0, 31], [0, -3], [0, 23], [43, 0], [0, -36], [-14, 14], [-21, 21], [49, 0], [18, 0], [45, 0], [24, -24], [-87, 0], [-9, 9], [0, 73], [0, -51], [11, -11], [0, 21], [0, 36], [0, 47], [1, 0], [0, -54], [16, -16], [-51, 51], [5, 0], [-34, 0], [-75, 0], [11, -11], [39, 0], [0, -50], [33, -33], [18, -18], [-81, 81], [0, -17], [48, 0], [-34, 34], [0, -3], [0, -75], [0, 71], [27, -27], [-59, 0], [-73, 0], [66, 0], [-88, 0], [21, 0], [0, 2], [0, -56], [35, -35], [63, -63], [-66, 0], [-23, 23], [0, 51], [0, -73], [63, 0], [-20, 20], [12, 0], [-82, 0], [75, 0], [-85, 85], [0, 20], [-61, 0], [0, 39], [-27, 27], [0, 1], [51, -51], [24, 0], [2, -2], [73, -73], [0, 93], [0, 42], [31, 0], [-14, 14], [15, -15], [-35, 0], [-25, 0], [-6, 0], [45, 0], [-60, 60], [-15, 0], [0, 75], [0, 61], [33, -33], [36, -36], [51, -51], [68, -68], [37, -37], [0, 21], [-68, 68], [-58, 0], [-75, 0], [0, -52], [15, 0], [0, 51], [0, 17], [0, 22], [0, -9], [-36, 0], [-58, 58], [0, -78], [0, -85], [39, 0], [-57, 57], [-16, 0], [-51, 0], [-91, 91], [-24, 24], [-1, 0], [0, -13], [0, 76], [63, -63], [0, 37], [21, -21], [24, -24], [-73, 73], [-81, 81], [0, 61], [92, -92], [0, -27], [-5, 5], [0, -33], [-59, 0], [0, -66], [5, 0], [-45, 0], [90, -90], [0, 20], [-25, 0], [0, -33], [0, 65], [-15, 15], [35, -35], [-33, 33], [24, 0], [13, -13], [-59, 59], [0, 30], [0, -24], [61, 0], [-42, 0], [57, 0], [-39, 0], [0, 74], [0, 87], [-45, 45], [-38, 0], [0, 57], [-33, 33], [90, 0], [-52, 52], [-93, 0]]
